==> 0.0.27.tar.gz: lib/http/dio_utils.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:dio/adapter.dart';
import 'package:dio/dio.dart';

import 'base_response_bean.dart';
import 'http_error.dart';

/// 请求方法.
class Method {
  static final String get = "GET";
  static final String post = "POST";
  static final String put = "PUT";
  static final String head = "HEAD";
  static final String delete = "DELETE";
  static final String patch = "PATCH";
}


class HttpConfig {
  /// constructor.
  HttpConfig({
    this.code,
    this.msg,
    this.data,
    this.options,
    this.pem,
    this.pKCSPath,
    this.pKCSPwd,
    this.baseUrl,
    this.successCode,
  });


  /// BaseResp [int code]字段 key, 默认：errorCode.
  String code;

  /// BaseResp [String msg]字段 key, 默认：errorMsg.
  String msg;

  /// BaseResp [T data]字段 key, 默认：data.
  String data;

  /// Options.
  BaseOptions options;

  ///服务器成功返回的code
  String successCode = '0';

  /// 详细使用请查看dio官网 https://github.com/flutterchina/dio/blob/flutter/README-ZH.md#Https证书校验.
  /// PEM证书内容.
  String pem;

  /// 详细使用请查看dio官网 https://github.com/flutterchina/dio/blob/flutter/README-ZH.md#Https证书校验.
  /// PKCS12 证书路径.
  String pKCSPath;

  /// 详细使用请查看dio官网 https://github.com/flutterchina/dio/blob/flutter/README-ZH.md#Https证书校验.
  /// PKCS12 证书密码.
  String pKCSPwd;


  /// 服务器地址
  String baseUrl;
}



class DioUtils {

  //服务器路径

  static DioUtils instance = new DioUtils();

  BaseOptions _options;

  Dio _dio;

  bool _isDeBug = true;

  HttpConfig httpConfig;

  InterceptorsWrapper interceptorsWrapper;

  static DioUtils getInstance(){
    return instance;
  }






  ///-----------------------------------------在这里自定义服务器返回的json字段-------------------------------------
  String _msgKey = 'message';
  String _codeKey = "code";
  String _dataKey = 'data';
  String _successCode = '0';


  ///-----------------------------------------证书相关信息-------------------------------------
  /// PEM证书内容.
  String _pem;

  /// PKCS12 证书路径.
  String _pKCSPath;

  /// PKCS12 证书密码.
  String _pKCSPwd;


  /// set Config.
  void setConfig(HttpConfig config) {
    this.httpConfig = config;
    _codeKey = config.code ?? _codeKey;
    _msgKey = config.msg ?? _msgKey;
    _dataKey = config.data ?? _dataKey;
    _successCode = config.successCode ?? _successCode;
    _mergeOption(config);
    _pem = config.pem ?? _pem;
    if (_dio != null) {
      if (_pem != null) {
        (_dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate = (HttpClient client) {
          client.badCertificateCallback =
              (X509Certificate cert, String host, int port) {
            if (cert.pem == _pem) {
              // 证书一致，则放行
              return true;
            }
            return false;
          };
        };
      }
      if (_pKCSPath != null) {
        (_dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate = (HttpClient client) {
          SecurityContext sc = new SecurityContext();
          //file为证书路径
          sc.setTrustedCertificates(_pKCSPath, password: _pKCSPwd);
          HttpClient httpClient = new HttpClient(context: sc);
          return httpClient;
        };
      }
    }
  }



  /// merge Option.
  void _mergeOption(HttpConfig httpConfig) {
    BaseOptions opt = httpConfig.options;
    if(opt != null){
      _options.method = opt.method ?? _options.method;
      _options.headers = (new Map.from(_options.headers))..addAll(opt.headers);
      _options.sendTimeout = opt.sendTimeout ?? _options.sendTimeout;
      _options.receiveTimeout = opt.receiveTimeout ?? _options.receiveTimeout;
      _options.responseType = opt.responseType ?? _options.responseType;
      _options.extra = (new Map.from(_options.extra))..addAll(opt.extra);
      _options.contentType = opt.contentType ?? _options.contentType;
      _options.validateStatus = opt.validateStatus ?? _options.validateStatus;
      _options.followRedirects = opt.followRedirects ?? _options.followRedirects;
    }

    if(httpConfig.baseUrl != null){
      _dio.options.baseUrl = httpConfig.baseUrl?? _dio.options.baseUrl;
    }

  }



  ///-----------------------------------------工具类初始化设置-------------------------------------

  DioUtils(){
    //基础配置
    _options = getDefOptions();
    _dio = new Dio(_options);
  }



  /// get Def Options.
  BaseOptions getDefOptions() {
    if(_options == null){
      _options = new BaseOptions();
      _options.contentType = Headers.formUrlEncodedContentType;
      _options.connectTimeout = 1000 * 5;
      _options.receiveTimeout = 1000 * 30;
    }
    return _options;
  }

  ///修改默认url
  void changeBaseUrl(String baseUrl){
    _dio.options.baseUrl = baseUrl;
  }


  ///-----------------------------------------请求方法封装-------------------------------------

  Options checkOptions(method, options) {
    options ??= Options();
    options.method = method;
    return options;
  }

  // ignore: missing_return
  Future<BaseResp<T>> request<T>(String method, String path, {data, Options options, CancelToken cancelToken, Function onError}) async{
    Response response;
    try{
      response = await _dio.request(
          path,
          data: data,
          options: checkOptions(method, options),
          cancelToken: cancelToken
      );


      _printHttpLog(response);
      String _code;
      String _msg;
      T _data;
      if (response.statusCode == HttpStatus.ok || response.statusCode == HttpStatus.created){
        if(response.data is Map){
          _code = response.data[_codeKey].toString();
          _msg = response.data[_msgKey].toString();
          _data = response.data[_dataKey];
          if(_code != null && _code != _successCode){
            if(onError != null){
              onError.call(HttpError(_code.toString(), _msg));
            }else{
              throw (HttpError(_code.toString(), _msg));
            }
          }
        }else if(response.data is List){
          return BaseResp(_code, _msg, _data, response);
        }else{
          Map<String, dynamic> _dataMap = _decodeData(response);
          _code = _dataMap[_codeKey];
          _msg = _dataMap[_msgKey];
          _data = _dataMap[_dataKey];
          if(_code != _successCode){
            throw (HttpError(_code.toString(), _msg));
          }
        }
        return BaseResp(_code, _msg, _data, response);
      }
    }on DioError catch (e) {
      if(onError != null){
        onError.call(HttpError.dioError(e));
      }else{
        throw (HttpError.dioError(e));
      }
    }catch(e){
      if(onError != null){
        onError.call(HttpError(HttpError.UNKNOWN, e.toString()));
      }else{
        throw (HttpError(HttpError.UNKNOWN, e.toString()));
      }
    }

  }





  /// decode response data.
  Map<String, dynamic> _decodeData(Response response) {
    if (response == null ||
        response.data == null ||
        response.data.toString().isEmpty) {
      return new Map();
    }
    return json.decode(response.data.toString());
  }



  ///-----------------------------------------dio部分设置-------------------------------------
  void openDebug(){
    _isDeBug = true;
  }

  void closeDebug(){
    _isDeBug = false;
  }


  ///用于添加拦截器,可以设置全局请求处理
  void setinterceptors(InterceptorsWrapper interceptorsWrapper){
    _dio.interceptors.add(interceptorsWrapper);
  }



  void setCookie(String cookie){
    Map<String, dynamic> _headers = new Map();
    _headers["Cookie"] = cookie;
    _dio.options.headers.addAll(_headers);
  }


  ///-----------------------------------------debug打印方法-------------------------------------
  /// print Http Log.
  void _printHttpLog(Response response) {
    if (!_isDeBug) {
      return;
    }
    try {
      print("----------------Http Log----------------" +
          "\n[statusCode]:   " +
          response.statusCode.toString() +
          "\n[request   ]:   " +
          _getOptionsStr(response.request));
      _printDataStr("reqdata ", response.request.data);
      _printDataStr("response", response.data);
      print("----------------end----------------");
    } catch (ex) {
      print("Http Log" + " error......");
    }
  }


  /// get Options Str.
  String _getOptionsStr(RequestOptions request) {
    return "method: " +
        request.method +
        "  baseUrl: " +
        request.baseUrl +
        "  path: " +
        request.path;
  }


  /// print Data Str.
  void _printDataStr(String tag, Object value) {
    String da = value.toString();
    while (da.isNotEmpty) {
      if (da.length > 512) {
        print("[$tag  ]:   " + da.substring(0, 512));
        da = da.substring(512, da.length);
      } else {
        print("[$tag  ]:   " + da);
        da = "";
      }
    }
  }
}

==> 0.1.50.tar.gz: lib/http/dio_utils.dart <==
import 'dart:convert';
import 'dart:io';

import 'package:connectivity/connectivity.dart';
import 'package:dio/adapter.dart';
import 'package:dio/dio.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter_mylibrary/flutter_mylibrary.dart';
import 'package:flutter_mylibrary/http/base_response_bean.dart';
import 'package:flutter_mylibrary/http/exception/http_error.dart';
import 'package:flutter_mylibrary/provider/view_state_model.dart';
import 'package:flutter_mylibrary/utils/utils.dart';
import 'retry/dioconnectivity_request_retrier.dart';
import 'retry/retry_onconnectionchange_interceptor.dart';


/// 请求方法.
class Method {
  static final String get = "GET";
  static final String post = "POST";
  static final String put = "PUT";
  static final String head = "HEAD";
  static final String delete = "DELETE";
  static final String patch = "PATCH";
}


class HttpConfig {
  /// constructor.
  HttpConfig({
    this.code,
    this.msg,
    this.data,
    this.options,
    this.pem,
    this.pKCSPath,
    this.pKCSPwd,
    this.baseUrl,
    this.successCode,
  });


  /// BaseResp [int code]字段 key, 默认：errorCode.
  String code;

  /// BaseResp [String msg]字段 key, 默认：errorMsg.
  String msg;

  /// BaseResp [T data]字段 key, 默认：data.
  String data;

  /// Options.
  BaseOptions options;

  ///服务器成功返回的code
  String successCode = '0';

  /// 详细使用请查看dio官网 https://github.com/flutterchina/dio/blob/flutter/README-ZH.md#Https证书校验.
  /// PEM证书内容.
  String pem;

  /// 详细使用请查看dio官网 https://github.com/flutterchina/dio/blob/flutter/README-ZH.md#Https证书校验.
  /// PKCS12 证书路径.
  String pKCSPath;

  /// 详细使用请查看dio官网 https://github.com/flutterchina/dio/blob/flutter/README-ZH.md#Https证书校验.
  /// PKCS12 证书密码.
  String pKCSPwd;


  /// 服务器地址
  String baseUrl;

}




class DioUtils {

  //服务器路径

  static DioUtils instance = new DioUtils();

  BaseOptions _options;

  Dio _dio;

  bool _isDeBug = true;

  HttpConfig httpConfig;

  InterceptorsWrapper interceptorsWrapper;

  static DioUtils getInstance(){
    return instance;
  }


  Function showLoading;

  Function onError;

  Function onFinish;

  Function onStart;


  ///-----------------------------------------在这里自定义服务器返回的json字段-------------------------------------
  String _msgKey = 'message';
  String _codeKey = "code";
  String _dataKey = 'data';
  String _successCode = '0';


  ///-----------------------------------------证书相关信息-------------------------------------
  /// PEM证书内容.
  String _pem;

  /// PKCS12 证书路径.
  String _pKCSPath;

  /// PKCS12 证书密码.
  String _pKCSPwd;



  /// set Config.
  void setConfig(HttpConfig config) {
    this.httpConfig = config;
    _codeKey = config.code ?? _codeKey;
    _msgKey = config.msg ?? _msgKey;
    _dataKey = config.data ?? _dataKey;
    _successCode = config.successCode ?? _successCode;
    _mergeOption(config);
    _pem = config.pem ?? _pem;
    if (_dio != null) {

      (_dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate = (HttpClient client) {
        client.badCertificateCallback = (X509Certificate cert, String host, int port) {
          if (_pem != null) {
            if (cert.pem == _pem) {
              // 证书一致，则放行
              return true;
            }else{
              return false;
            }
          }else{
            return true;
          }
        };
      };
      if (_pKCSPath != null) {
        (_dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate = (HttpClient client) {
          SecurityContext sc = new SecurityContext();
          //file为证书路径
          sc.setTrustedCertificates(_pKCSPath, password: _pKCSPwd);
          HttpClient httpClient = new HttpClient(context: sc);
          return httpClient;
        };
      }
    }
  }



  /// merge Option.
  void _mergeOption(HttpConfig httpConfig) {
    BaseOptions opt = httpConfig.options;
    if(opt != null){
      _options.method = opt.method ?? _options.method;
      _options.headers = (new Map.from(_options.headers))..addAll(opt.headers);
      _options.sendTimeout = opt.sendTimeout ?? _options.sendTimeout;
      _options.receiveTimeout = opt.receiveTimeout ?? _options.receiveTimeout;
      _options.responseType = opt.responseType ?? _options.responseType;
      _options.extra = (new Map.from(_options.extra))..addAll(opt.extra);
      _options.contentType = opt.contentType ?? _options.contentType;
      _options.validateStatus = opt.validateStatus ?? _options.validateStatus;
      _options.followRedirects = opt.followRedirects ?? _options.followRedirects;
    }

    if(httpConfig.baseUrl != null){
      _dio.options.baseUrl = httpConfig.baseUrl?? _dio.options.baseUrl;
    }

  }



  ///-----------------------------------------工具类初始化设置-------------------------------------

  DioUtils(){
    //基础配置
    _options = getDefOptions();
    _dio = new Dio(_options);
  }



  /// get Def Options.
  BaseOptions getDefOptions() {
    if(_options == null){
      _options = new BaseOptions();
      _options.contentType = Headers.formUrlEncodedContentType;
      _options.connectTimeout = 1000 * 5;
      _options.receiveTimeout = 1000 * 30;
    }
    return _options;
  }

  ///修改默认url
  void changeBaseUrl(String baseUrl){
    _dio.options.baseUrl = baseUrl;
  }



  ///-----------------------------------------请求方法封装-------------------------------------

  Options checkOptions(method, options) {
    options ??= Options();
    options.method = method;
    return options;
  }

  // ignore: missing_return
  Future<BaseResp<T>> request<T>(String method, String path, {bool publicMethod = true, ViewStateModel viewStateModel, data, Options options, CancelToken cancelToken, Function onStart, Function onError, Function onFinish, String datakey}) async{
    if (onStart != null){
      onStart();
    }else if(this.onStart != null){
      this.onStart();
    }

    Response response;
    String _code;
    String _msg;
    T _data;
    try{
      if (method == Method.get) {
        response = await _dio.request(
            path,
            queryParameters: data,
            options: checkOptions(method, options),
            cancelToken: cancelToken
        );
      } else{
        response = await _dio.request(
            path,
            data: data,
            options: checkOptions(method, options),
            cancelToken: cancelToken
        );
      }



      _printHttpLog(response);

      if (response.statusCode == HttpStatus.ok || response.statusCode == HttpStatus.created){
        if(response.data is Map){
          _code = response.data[_codeKey].toString();
          _msg = response.data[_msgKey].toString();

          if(_code != null && _code != _successCode){
            throw (HttpError(_code.toString(), _msg));
          }else{
            _data = datakey != null? response.data[datakey] : response.data[_dataKey];
          }
        }else if(response.data is List){
          if (onFinish != null) {
            onFinish();
          }else if(this.onFinish != null){
            this.onFinish(response);
          }
          if (isNotEmpty(viewStateModel)) {
            viewStateModel.setIdle();
          }
          return BaseResp(_code, _msg, _data, response);
        }else{

          Map<String, dynamic> _dataMap = _decodeData(response);
          _code = _dataMap[_codeKey].toString();
          _msg = _dataMap[_msgKey].toString();
          _data = datakey != null? _dataMap[datakey] : _dataMap[_dataKey];
          if(_code != _successCode){
            throw (HttpError(_code.toString(), _msg));
          }
        }
        if (onFinish != null) {
          onFinish();
        }else if(this.onFinish != null){
          this.onFinish(response);
        }
        if (isNotEmpty(viewStateModel)) {
          viewStateModel.setIdle();
        }
        return BaseResp(_code, _msg, _data, response);
      }
    }catch(e, s){
      var error;
      if (e is DioError) {
        error = HttpError.dioError(e);
      }else{
        error = e;
      }


      if (isNotEmpty(viewStateModel)) {
        viewStateModel.setError(error, s);
      }else{
        if (publicMethod) {
          if(onError != null){
            onError.call(error);
          }else if (this.onError != null) {
            this.onError.call(error);
          }
        }
      }

      if (isNotEmpty(onFinish)) {
        onFinish();
      } else if(this.onFinish != null){
        this.onFinish(response);
      }
      return Future.error(error);
    }
  }




  /// decode response data.
  Map<String, dynamic> _decodeData(Response response) {
    if (response == null ||
        response.data == null ||
        response.data.toString().isEmpty) {
      return new Map();
    }
    return json.decode(response.data.toString());
  }



  void fileUpload(String path, Map<String, dynamic> map, {Function onSendProgress, Function onStart, Function onError, Function onFinish}) async{
    FormData formData = FormData.fromMap(map);
    Response response;
    try{
      response = await _dio.post(
          path,
          data: formData,
          onSendProgress: (int sent, int total){
            if (onSendProgress != null) {
              onSendProgress.call(sent, total);
            }
          }
      );
    }on DioError catch(e){
      throw HttpError.dioError(e);
    }catch(e){
      if (e is HttpError) {
        if(onError != null){
          onError.call(e);
        }else if (this.onError != null) {
          this.onError.call(e);
        }

        if (isNotEmpty(onFinish)) {
          onFinish();
        } else if(this.onFinish != null){
          this.onFinish();
        }
      }
    }
  }



  ///-----------------------------------------dio部分设置-------------------------------------
  void openDebug(){
    _isDeBug = true;
  }

  void closeDebug(){
    _isDeBug = false;
  }


  ///用于添加拦截器,可以设置全局请求处理
  void setinterceptors(InterceptorsWrapper interceptorsWrapper, {bool replace = false}){
    if (replace) {
      if(_dio.interceptors.contains(this.interceptorsWrapper)){
        _dio.interceptors.remove(this.interceptorsWrapper);
      }
    }
    this.interceptorsWrapper = interceptorsWrapper;
    _dio.interceptors.add(interceptorsWrapper);
  }


  void enableAutoCookie() async{
    Directory temporaryDirectory = await getTemporaryDirectory();
    _dio.interceptors.add(CookieManager(
        PersistCookieJar(dir: temporaryDirectory.path)));
  }

  void enableRetry(){
    _dio.interceptors.add(
      RetryOnConnectionChangeInterceptor(
        requestRetrier: DioConnectivityRequestRetrier(
          dio: _dio,
          connectivity: Connectivity(),
        ),
      ),
    );
  }




  void setCookie(String cookie){
    Map<String, dynamic> _headers = new Map();
    _headers["Cookie"] = cookie;
    _dio.options.headers.addAll(_headers);
  }


  ///-----------------------------------------debug打印方法-------------------------------------
  /// print Http Log.
  void _printHttpLog(Response response) {
    if (!_isDeBug) {
      return;
    }
    try {
      debugPrint("----------------Http Log----------------" +
          "\n[statusCode]:   " +
          response.statusCode.toString() +
          "\n[request   ]:   " +
          _getOptionsStr(response.request));
      _printDataStr("header ", response.request.headers);
      _printDataStr("reqdata ", response.request.data);
      _printDataStr("response", response.data);
      debugPrint("----------------end----------------");
    } catch (ex) {
      debugPrint("Http Log" + " error......");
    }
  }


  /// get Options Str.
  String _getOptionsStr(RequestOptions request) {
    return "method: " +
        request.method +
        "  baseUrl: " +
        request.baseUrl +
        "  path: " +
        request.path;
  }


  /// print Data Str.
  void _printDataStr(String tag, Object value) {
    String da = value.toString();
    while (da.isNotEmpty) {
      if (da.length > 512) {
        debugPrint("[$tag  ]:   " + da.substring(0, 512));
        da = da.substring(512, da.length);
      } else {
        debugPrint("[$tag  ]:   " + da);
        da = "";
      }
    }
  }
}
